@@ -35,10 +35,13 @@
 #include "config.h"
 #endif
 
+#include "clutter-actor-private.h"
 #include "clutter-debug.h"
+#include "clutter-device-manager-private.h"
 #include "clutter-enum-types.h"
 #include "clutter-input-device.h"
 #include "clutter-private.h"
+#include "clutter-stage-private.h"
 
 enum
 {
@@ -510,8 +513,12 @@ _clutter_input_device_update (ClutterInputDevice *device)
   g_return_val_if_fail (device->device_type == CLUTTER_POINTER_DEVICE, NULL);
 
   stage = device->stage;
-  if (stage == NULL)
-    return NULL;
+  if (G_UNLIKELY (stage == NULL))
+    {
+      CLUTTER_NOTE (EVENT, "No stage defined for device '%s'",
+                    clutter_input_device_get_device_name (device));
+      return NULL;
+    }
 
   clutter_input_device_get_device_coords (device, &x, &y);
 
