@@ -24,10 +24,12 @@
 #include "clutter-osx.h"
 #include "clutter-stage-osx.h"
 #include "clutter-backend-osx.h"
-#import <AppKit/AppKit.h>
 
-#include <clutter/clutter-debug.h>
-#include <clutter/clutter-private.h>
+#include "clutter-debug.h"
+#include "clutter-private.h"
+#include "clutter-stage-private.h"
+
+#import <AppKit/AppKit.h>
 
 static void clutter_stage_window_iface_init (ClutterStageWindowIface *iface);
 
@@ -400,7 +402,7 @@ clutter_stage_osx_resize (ClutterStageWindow *stage_window,
   ClutterActor *actor = clutter_stage_osx_get_wrapper (stage_window);
 
   guint min_width, min_height;
-  clutter_stage_get_minimum_size (actor,
+  clutter_stage_get_minimum_size (CLUTTER_STAGE (actor),
                                   &min_width,
                                   &min_height);
   [self->window setContentMinSize:NSMakeSize(min_width, min_height)];
